@@ -1,6 +1,6 @@
 /********************************************************************************
 * ReactPhysics3D physics library, http://www.reactphysics3d.com                 *
-* Copyright (c) 2010-2020 Daniel Chappuis                                       *
+* Copyright (c) 2010-2022 Daniel Chappuis                                       *
 *********************************************************************************
 *                                                                               *
 * This software is provided 'as-is', without any express or implied warranty.   *
@@ -41,35 +41,35 @@ class HalfEdgeStructure {
 
     public:
 
-        using VerticesPair = Pair<uint, uint>;
+        using VerticesPair = Pair<uint32, uint32>;
 
         /// Edge
         struct Edge {
-            uint vertexIndex;       // Index of the vertex at the beginning of the edge
-            uint twinEdgeIndex;     // Index of the twin edge
-            uint faceIndex;         // Adjacent face index of the edge
-            uint nextEdgeIndex;     // Index of the next edge
+            uint32 vertexIndex;       // Index of the vertex at the beginning of the edge
+            uint32 twinEdgeIndex;     // Index of the twin edge
+            uint32 faceIndex;         // Adjacent face index of the edge
+            uint32 nextEdgeIndex;     // Index of the next edge
         };
 
         /// Face
         struct Face {
-            uint edgeIndex;             // Index of an half-edge of the face
-            List<uint> faceVertices;	// Index of the vertices of the face
+            uint32 edgeIndex;             // Index of an half-edge of the face
+            Array<uint32> faceVertices;	// Index of the vertices of the face
 
             /// Constructor
-            Face(MemoryAllocator& allocator) : faceVertices(allocator) {}
+            Face(MemoryAllocator& allocator) : edgeIndex(0), faceVertices(allocator) {}
 
             /// Constructor
-            Face(List<uint> vertices) : faceVertices(vertices) {}
+            Face(Array<uint32> vertices) : edgeIndex(0), faceVertices(vertices) {}
         };
 
         /// Vertex
         struct Vertex {
-            uint vertexPointIndex;  // Index of the vertex point in the origin vertex array
-            uint edgeIndex;         // Index of one edge emanting from this vertex
+            uint32 vertexPointIndex;  // Index of the vertex point in the origin vertex array
+            uint32 edgeIndex;         // Index of one edge emanting from this vertex
 
             /// Constructor
-            Vertex(uint vertexCoordsIndex) : vertexPointIndex(vertexCoordsIndex) { }
+            Vertex(uint32 vertexCoordsIndex) : vertexPointIndex(vertexCoordsIndex), edgeIndex(0) { }
         };
 
     private:
@@ -78,19 +78,19 @@ class HalfEdgeStructure {
         MemoryAllocator& mAllocator;
 
         /// All the faces
-        List<Face> mFaces;
+        Array<Face> mFaces;
 
         /// All the vertices
-        List<Vertex> mVertices;
+        Array<Vertex> mVertices;
 
         /// All the half-edges
-        List<Edge> mEdges;
+        Array<Edge> mEdges;
 
     public:
 
         /// Constructor
-        HalfEdgeStructure(MemoryAllocator& allocator, uint facesCapacity, uint verticesCapacity,
-                          uint edgesCapacity) :mAllocator(allocator), mFaces(allocator, facesCapacity),
+        HalfEdgeStructure(MemoryAllocator& allocator, uint32 facesCapacity, uint32 verticesCapacity,
+                          uint32 edgesCapacity) :mAllocator(allocator), mFaces(allocator, facesCapacity),
                           mVertices(allocator, verticesCapacity), mEdges(allocator, edgesCapacity) {}
 
         /// Destructor
@@ -100,28 +100,28 @@ class HalfEdgeStructure {
         void init();
 
         /// Add a vertex
-        uint addVertex(uint vertexPointIndex);
+        uint32 addVertex(uint32 vertexPointIndex);
 
         /// Add a face
-        void addFace(List<uint> faceVertices);
+        void addFace(Array<uint32> faceVertices);
 
         /// Return the number of faces
-        uint getNbFaces() const;
+        uint32 getNbFaces() const;
 
         /// Return the number of half-edges
-        uint getNbHalfEdges() const;
+        uint32 getNbHalfEdges() const;
 
         /// Return the number of vertices
-        uint getNbVertices() const;
+        uint32 getNbVertices() const;
 
         /// Return a given face
-        const Face& getFace(uint index) const;
+        const Face& getFace(uint32 index) const;
 
         /// Return a given edge
-        const Edge& getHalfEdge(uint index) const;
+        const Edge& getHalfEdge(uint32 index) const;
 
         /// Return a given vertex
-        const Vertex& getVertex(uint index) const;
+        const Vertex& getVertex(uint32 index) const;
 
 };
 
@@ -129,18 +129,18 @@ class HalfEdgeStructure {
 /**
  * @param vertexPointIndex Index of the vertex in the vertex data array
  */
-inline uint HalfEdgeStructure::addVertex(uint vertexPointIndex) {
+RP3D_FORCE_INLINE uint32 HalfEdgeStructure::addVertex(uint32 vertexPointIndex) {
     Vertex vertex(vertexPointIndex);
     mVertices.add(vertex);
-    return mVertices.size() - 1;
+    return static_cast<uint32>(mVertices.size()) - 1;
 }
 
 // Add a face
 /**
- * @param faceVertices List of the vertices in a face (ordered in CCW order as seen from outside
+ * @param faceVertices Array of the vertices in a face (ordered in CCW order as seen from outside
  *                     the polyhedron
  */
-inline void HalfEdgeStructure::addFace(List<uint> faceVertices) {
+RP3D_FORCE_INLINE void HalfEdgeStructure::addFace(Array<uint32> faceVertices) {
 
     // Create a new face
     Face face(faceVertices);
@@ -151,31 +151,31 @@ inline void HalfEdgeStructure::addFace(List<uint> faceVertices) {
 /**
  * @return The number of faces in the polyhedron
  */
-inline uint HalfEdgeStructure::getNbFaces() const {
-    return static_cast<uint>(mFaces.size());
+RP3D_FORCE_INLINE uint32 HalfEdgeStructure::getNbFaces() const {
+    return static_cast<uint32>(mFaces.size());
 }
 
 // Return the number of edges
 /**
  * @return The number of edges in the polyhedron
  */
-inline uint HalfEdgeStructure::getNbHalfEdges() const {
-    return static_cast<uint>(mEdges.size());
+RP3D_FORCE_INLINE uint32 HalfEdgeStructure::getNbHalfEdges() const {
+    return static_cast<uint32>(mEdges.size());
 }
 
 // Return the number of vertices
 /**
  * @return The number of vertices in the polyhedron
  */
-inline uint HalfEdgeStructure::getNbVertices() const {
-    return static_cast<uint>(mVertices.size());
+RP3D_FORCE_INLINE uint32 HalfEdgeStructure::getNbVertices() const {
+    return static_cast<uint32>(mVertices.size());
 }
 
 // Return a given face
 /**
  * @return A given face of the polyhedron
  */
-inline const HalfEdgeStructure::Face& HalfEdgeStructure::getFace(uint index) const {
+RP3D_FORCE_INLINE const HalfEdgeStructure::Face& HalfEdgeStructure::getFace(uint32 index) const {
     assert(index < mFaces.size());
     return mFaces[index];
 }
@@ -184,7 +184,7 @@ inline const HalfEdgeStructure::Face& HalfEdgeStructure::getFace(uint index) con
 /**
  * @return A given edge of the polyhedron
  */
-inline const HalfEdgeStructure::Edge& HalfEdgeStructure::getHalfEdge(uint index) const {
+RP3D_FORCE_INLINE const HalfEdgeStructure::Edge& HalfEdgeStructure::getHalfEdge(uint32 index) const {
     assert(index < mEdges.size());
     return mEdges[index];
 }
@@ -193,7 +193,7 @@ inline const HalfEdgeStructure::Edge& HalfEdgeStructure::getHalfEdge(uint index)
 /**
  * @return A given vertex of the polyhedron
  */
-inline const HalfEdgeStructure::Vertex& HalfEdgeStructure::getVertex(uint index) const {
+RP3D_FORCE_INLINE const HalfEdgeStructure::Vertex& HalfEdgeStructure::getVertex(uint32 index) const {
     assert(index < mVertices.size());
     return mVertices[index];
 }
